; -----------------------------------------------------------------------------
; Especificaciones: Listado de variables para ZX Spectrum 48K.
; Descripción general: Se listan todas aqellas variables que
; enten almacenadas en memoria con sus correspondientes contenidos.
; Longitud: 509.
; Entrada: Ningún requerimiento.
; Salida: Ningún requerimiento.
;
; Tipos de variables:
;
;	- 011 Numérica de una sola letra		; 6 bytes por variable
;	- 101 Numérica de más de una letra		; 8 bytes por variable
; 	- 100 Matriz numérica					; 8 bytes + (5 bytes * longitud)
;
;	- 010 Alfanumérica						; 3 bytes + longitud cadena
; 	- 110 Matriz alfanumérica				; 8 bytes + (dimensión1 * dimensión2)
;
;	- 111 Control de bucle FOR-NEXT			; 19 bytes
; -----------------------------------------------------------------------------

org		$fd5b	; 48K	64859

; -----------------------------------------------------------------------------
; Rutinas de la ROM
; -----------------------------------------------------------------------------
analn		equ $2ab2	; Almacena en pila del calculador para printn, 
						; registros A, E, D, C, B, en ese orden
cls			equ $0d6b	; Limpia la pantalla
libre1		equ	$5cb0	; Dirección libre donde se puede guardar un byte
libre2		equ $5cb1	; Dirección libre donde se puede guardar un byte
open		equ $1601	; Abre el canal
printn		equ $2de3	; Imprime el número de la pila del calculador
prints		equ $203c	; Imprime una cadena, DE = posición de la cadena, BC = longitud

; -----------------------------------------------------------------------------
; Variables de sistema
; -----------------------------------------------------------------------------
vars		equ $5c4b	; Dirección de la primera variable del programa Basic

; -----------------------------------------------------------------------------
; Punto de entrada
; -----------------------------------------------------------------------------
Inicio:
	call	cls					; Borra la pantalla
	ld		a, $02
	call	open				; Abre el canal 2, pantalla
	ld		hl, [vars]			; HL = dirección de la primera variable del programa

; -----------------------------------------------------------------------------
; Evalúa que tipo de variable es
; -----------------------------------------------------------------------------
SigVar:
	ld		a, [hl]
	cp		$80					; Comprueba si se ha llegado al fin de las variables
	ret		z					; Si es así, sale
	bit		$07, a				; Comprueba el estado del bit 7
	jr		nz, Bit7_1			; Si está a 1, salta
	bit		$05, a				; Comprueba el estado del bit 5
	jp		nz, VarA			; Si está a 1, 011 es de tipo numérica de una sola letra
	jp		VarAS				; Si esrá a 0, 010 es de tipo alfanumérica

	Bit7_1:
		bit		$06, a			; Comprueba el estado del bit 6
		jr		nz, Bit6_1		; Si está a 1, salta
		bit		$05, a			; Comprueba el estado del bit 5
		jp		nz, VarAbc		; Si está a 1, 101 es de tipo numérica de más de una letra
		jp		VarA1			; Si está a 0, 100 Es de tipo matriz numérica

	Bit6_1:
		bit		$05, a			; Comprueba el estado del bit 5
		jp		nz, VarBuc		; Si está a 1, 111 Si está a uno es de tipo control de bucle FOR-NEXT

; -----------------------------------------------------------------------------
; 110 Matriz alfanumérica
; -----------------------------------------------------------------------------
VarAS1:
	; Nombre de la variable
	ld		a, [hl]				; A = valor apuntado por HL
	xor		$80					; A = código de carácter
	rst		$10					; Lo imprime
	ld		a, $24				; A = código de $
	rst		$10					; Lo imprime
	inc		hl
	ld		e, [hl]
	inc		hl
	ld		d, [hl]				; DE = número de dimensiones + elementos + 1
	push	de					; Preserva DE
	inc		hl
	ld		b, [hl]				; B = número de dimensiones
	push	bc					; Preserva BC
	ld		a, $28				; A = código de (
	rst		$10					; Lo imprime
	inc		hl

	OtraDim1:
		push	bc				; Preserva BC
		ld		a, $00
		ld		e, a			; A = B = E = 0
		ld		d, [hl]			; Byte bajo
		inc		hl
		ld		c, [hl]			; Byte alto. D y C contienen la dimensión
		inc		hl
		ld		b, a
		push	hl				; Preserva HL
		call	analn			; Almacena A, E, D, C, y B en la pila del calculador
		call	printn			; Imprime el número de la pila del calculador
		pop		hl				; Recupera HL
		pop		bc				; Recupera BC
		ld		a, b
		cp		$01				; Comprueba si A/B es 1
		jr		z, FinDim1		; Si es 1 no lo imprime
		ld		a, $2c			; A = código de ,
		rst		$10				; Lo imprime

	FinDim1:
		djnz	OtraDim1
		ld		a, $29			; A = código de )
		rst		$10				; Lo imprime
		dec		hl
		dec		hl				; HL = dirección de la última dimensión
		ld		c, [hl]
		inc		hl
		ld		b, [hl]			; BC = valor al que apunta HL
		inc		hl
		ld		[libre1], bc	; Lo carga en memoria libre1 y libre2
		pop		bc				; Recupera el número de dimensiones
		pop		de				; Recupera la longitud
		dec		de

	EncLon1:
		; Número de elementos de la matriz
		dec		de
		dec		de				; Resta 2 a cada dimensión
		djnz	EncLon1			; Hasta que B = 0
								; DE = número de elementos de la matriz

	SigEle:
		; Siguiente elemento
		ld		bc, [libre1]	; Recupera la dirección de la última dimensión
		ld		a, $0d			; A = código ENTER
		rst		$10				; Lo imprime
	
	ContEle:
		; Elementos de la dimensión
		ld		a, d
		or		e
		dec		de
		jp		z, SigVar		; Si DE es cero, antes de DEC DE, siguiente variable
		ld		a, [hl]			; A = valor al que apunta HL
		inc		hl				; Apunta HL al siguiente byte
		rst		$10				; Imprime A
		dec		bc
		ld		a, b
		or		c
		jr		nz, ContEle		; Si BC <> 0 continua elemento
		jr		SigEle			; Si BC = 0 siguiente elemento

; -----------------------------------------------------------------------------
; 111 Control de bucle FOR-NEXT
; -----------------------------------------------------------------------------
VarBuc:
	ld		a, $14				; A = código INVERSE
	rst		$10					; Lo imprime
	ld		a, $01				; A = modo inverse 1
	rst		$10					; Lo imprime
	; Nombre de la variable de control
	ld		a, [hl]				; A = valor al que apunta HL
	xor		$a0					; A = código de carácter del valor
	rst		$10					; Lo imprime
	; Variable de control del bucle
	ld		de, Mens1			; DE = dirección del primer mensaje
	ld		bc, Mens2 - Mens1	; BC = longitud del primer mensaje
	call	prints				; Imprime el mensaje
	; Pasa exponente y mantisa a A, E, D, C, B del valor de la variable de control del bucle
	inc		hl
	ld		a, [hl]
	inc		hl
	ld		e, [hl]
	inc		hl
	ld		d, [hl]
	inc		hl
	ld		c, [hl]
	inc		hl
	ld		b, [hl]
	inc		hl
	push	hl
	call	analn				; Almacena A, E, D, C, y B en la pila del calculador
	call	printn				; Imprime el valor de la variable de control del bucle
	pop		hl

	;Límite del bucle
	ld		de, Mens2			; DE = dirección de segundo mensaje
	ld		bc, Mens3 - Mens2	; BC = longitud del primer mensaje
	call	prints				; Imprime el mensaje
	; Pasa exponente y mantisa a A, E, D, C, B del límite del bucle
	ld		a, [hl]
	inc		hl
	ld		e, [hl]
	inc		hl
	ld		d, [hl]
	inc		hl
	ld		c, [hl]
	inc		hl
	ld		b, [hl]
	inc		hl
	push	hl
	call	analn				; Almacena A, E, D, C, y B en la pila del calculador
	call	printn				; Imprime el valor del límite del bucle
	pop		hl
	; Paso del bucle
	ld		de, Mens3			; DE = dirección de segundo mensaje
	ld		bc, Mens4 - Mens3	; BC = longitud del primer mensaje
	call	prints				; Imprime el mensaje
	; Pasa exponente y mantisa a A, E, D, C, B del paso del bucle
	ld		a, [hl]
	inc		hl
	ld		e, [hl]
	inc		hl
	ld		d, [hl]
	inc		hl
	ld		c, [hl]
	inc		hl
	ld		b, [hl]
	inc		hl
	push	hl
	call	analn				; Almacena A, E, D, C, y B en la pila del calculador
	call	printn				; Imprime el paso del bucle
	pop		hl
	; Valor del número de línea
	ld		de, Mens4			; DE = dirección de segundo mensaje
	ld		bc, Mens5 - Mens4	; BC = longitud del primer mensaje
	call	prints				; Imprime el mensaje
	; Número de línea
	ld		a, $00
	ld		e, a
	ld		d, [hl]				; Byte bajo del número de línea
	inc		hl
	ld		c, [hl]				; Byte alto del número de línea
	inc		hl
	ld		b, a
	push	hl
	call	analn				; Almacena A, E, D, C, y B en la pila del calculador
	call	printn				; Imprime el número de línea
	pop		hl
	; Número de sentencia
	ld		de, Mens5			; DE = dirección de segundo mensaje
	ld		bc, MensF - Mens5	; BC = longitud del primer mensaje
	call	prints				; Imprime el mensaje
	; Número de sentencia siguiente al FOR
	ld		a, [hl]
	add		a, $30				; Cacula el código de caráter
	rst		$10					; Lo imprime
	inc		hl
	ld		a, $0d				; A = código de ENTER
	rst		$10					; Lo imprime
	jp		SigVar				; Siguiente variable
	
; -----------------------------------------------------------------------------
; 100 Matriz numérica
; -----------------------------------------------------------------------------
VarA1:
	; Nombre de la variable
	ld		a, [hl]
	xor		$c0					; Calcula el código del carácter
	rst		$10					; Lo imprime
	inc		hl
	ld		e, [hl]
	inc		hl
	ld		d, [hl]				; DE = número de dimensiones + elementos + 1
	push	de					; Preserva DE
	inc		hl
	ld		b, [hl]				; B = número de dimensiones
	push	bc					; Preserva BC
	ld		a, $28				; A = código de (
	rst		$10					; Lo imprime
	inc		hl

	OtraDim2:
		push	bc				; Preserva BC
		ld		a, $00
		ld		e, a
		ld		d, [hl]			; Byte bajo
		inc		hl
		ld		c, [hl]			; Byte alto. D y C contienen la dimensión
		inc		hl
		ld		b, a
		push	hl				; Preserva HL
		call	analn			; Almacena A, E, D, C y B en la pila del calculador
		call	printn			; Imprime el número de la pila del calculador
		pop		hl				; Recupera HL
		pop		bc				; Recupera BC
		ld		a, b
		cp		$01				; Comprueba si B = 1
		jr		z, FinDim2		; Si es 1, fin de la dimensión
		ld		a, $2c			; A = códido de ,
		rst		$10				; Lo imprime

	FinDim2:
		djnz	OtraDim2
		ld		a, $29			; A = código de )
		rst		$10				; Lo imprime
		pop		bc				; Recupera el número de dimensión
		pop		de				; Recupera la longitud
		dec		de

	EncLon2:
		dec		de
		dec		de				; Resta 2 a cada dimensión
		djnz	EncLon2			; Hasta que B = 0

	SigNum:
		ld		a, $0d			; A = código de ENTER
		rst		$10				; Lo imprime
		ld		a, d
		or		e				; Comprueba si DE = 0
		dec		de
		dec		de
		dec		de
		dec		de
		dec		de				; Resta 5 por cada elemento (1 número = 5 bytes)
		jp		z, SigVar		; Si DE es 0, siguiente variable
		push	de				; Preserva DE
		ld		a, [hl]
		inc		hl
		ld		e, [hl]
		inc		hl
		ld		d, [hl]
		inc		hl
		ld		c, [hl]
		inc		hl
		ld		b, [hl]			; Carga en A, E, D, C y B el exponente y mantisa
								; de un elemento del array	
		inc		hl
		push	hl				; Preserva HL
		call	analn			; Almacena A, E, D, C y B en la pila del calculador
		call	printn			; Imprime el número de la pila del calculador
		pop		hl				; Recupera HL
		pop		de				; Recupera la longitud
		jr		SigNum			; Siguiente número

; -----------------------------------------------------------------------------
; 101 Numérica de más de una letra
; -----------------------------------------------------------------------------
VarAbc:
	; Nombre de la variable
	ld		a, [hl]				; A = valor
	xor		$e0					; Calcula el código del carácter

	NoUlt:
		rst		$10				; Lo imprime
		inc		hl
		ld		a, [hl]			; A = valor
		bit		$07, a			; Comprueba si es el último
		jr		z, NoUlt		; Si no es el último, siguiente
		res		$07, a			; Pone el bit 7 a 0 para calcular el código del carácter
		rst		$10				; Lo imprime
		jp		Numerica		; Imprime el valor de la variable

; -----------------------------------------------------------------------------
; 010 Alfanumérica
; -----------------------------------------------------------------------------
VarAS:
	; Nombre de la variable
	ld		a, [hl]				; A = valor
	rst		$10					; Lo imprime
	ld		a, $24				; A = código de $
	rst		$10					; Lo imprime
	ld		a, $3d				; A = código de =
	rst		$10					; Lo imprime
	ld		a, $22				; A = código de "
	rst		$10					; Lo imprime
	inc		hl
	ld		c, [hl]
	inc		hl
	ld		b, [hl]				; BC = longitud de la cadena
	inc		hl

	SigCar:
		; Valor de la variable
		ld		a, b
		or		c				; Comprueba si BC = 0
		dec		bc
		jr		z, FinCar		; Si BC = 0, último carácter
		ld		a, [hl]			; A = valor
		inc		hl
		rst		$10				; Lo imprime
		jr		SigCar			; Siguiente carácter

	FinCar:
		ld		a, $22			; A = código de "
		rst		$10				; Lo imprime
		ld		a, $0d			; A = código de ENTER
		rst		$10				; Lo imprime
		jp		SigVar			; Siguiente variable

; -----------------------------------------------------------------------------
; 011 Numérica de una sola letra
; -----------------------------------------------------------------------------
VarA:
	; Nombre de la variable
	ld		a, [hl]				; HL = valor
	res		$05, a				; Pone a 0 del bit 5 para calcular el código del carácter
	rst		$10					; Lo imprime

	Numerica:
		; Valor de la variable
		ld		a, $3d			; A = código de =
		rst		$10				; Lo imprime
		inc		hl
		ld		a, [hl]
		inc		hl
		ld		e, [hl]
		inc		hl
		ld		d, [hl]
		inc		hl
		ld		c, [hl]
		inc		hl
		ld		b, [hl]			; A, E, D, C y B contienen exponte y mantisa
		inc		hl				; del número
		push	hl				; Preserva HL
		call	analn			; Almacena A, E, D, C y B en la pila del calculador
		call	printn			; Imprime el número de la pila del calculador
		pop		hl				; Recupera HL
		ld		a, $0d			; A = código de ENTER
		rst		$10				; Lo imprime
		jp		SigVar			; Siguiente variable

; -----------------------------------------------------------------------------
; Mensajes
; -----------------------------------------------------------------------------
Mens1:
	db		13, 20, 1				; INVERSE 1
	defm	"Valor"
	db		20, 0, 23, 12, 0		; INVERSE 0, TAB 15
Mens2:
	db		13, 20, 1				; INVERSE 1
	defm	"Limite"
	db		20, 0, 23, 12, 0		; INVERSE 0, TAB 15
Mens3:
	db		13, 20, 1				; INVERSE 1
	defm	"Paso"
	db		20, 0, 23, 12, 0		; INVERSE 0, TAB 15
Mens4:
	db		13, 20, 1				; INVERSE 1
	defm	"N. de linea"
	db		20, 0, 23, 12, 0		; INVERSE 0, TAB 15
Mens5:
	db		13, 20, 1				; INVERSE 1
	defm	"N. de orden"
	db		20, 0, 23, 12, 0		; INVERSE 0, TAB 15
MensF:
end		$fd5b	; 48K